require 'print/configuration'
require 'print/rest/address'
require 'httparty'

module Print
  class Client
    include HTTParty
    extend Configuration
    include Print::REST::Address

    attr_accessor *Configuration::VALID_CONFIG_KEYS

    def initialize(options={})
      merged_options = Print.options.merge(options)
      Configuration::VALID_CONFIG_KEYS.each do |key|
        send("#{key}=", merged_options[key])
      end
    end

    base_uri self.endpoint

    private

      def headers
        {
          "User-Agent" => self.user_agent,
          "Accept" => "application/io.tmd.v#{self.api_version}",
          "Authorization" => "Token token=\"#{self.api_key}\""
        }
      end

      def self.get_list(resource)
        self.get('/' + resource.to_s, headers: headers)
      end

      def self.get_by_id(resource, id)
        self.get('/' + resource.to_s + '/' + id, headers: headers)
      end

      def self.create(resource, hash)
        self.post('/' + resource.to_s, hash, headers: headers)
      end

  end
end