module Print
  module Configuration
    VALID_CONNECTION_KEYS = [:endpoint, :user_agent, :method].freeze
    VALID_OPTIONS_KEYS    = [:api_key, :format, :api_version].freeze
    VALID_CONFIG_KEYS     = VALID_CONNECTION_KEYS + VALID_OPTIONS_KEYS

    DEFAULT_ENDPOINT      = 'http://print.dev/api'
    DEFAULT_METHOD        = :get
    DEFAULT_USER_AGENT    = "Print API Ruby Gem #{Print::VERSION}".freeze
 
    DEFAULT_API_KEY       = nil
    DEFAULT_FORMAT        = :json
    DEFAULT_API_VERSION   = 1

    # Build accessor methods for every config options so we can do this, for example:
    #   Awesome.format = :xml
    attr_accessor *VALID_CONFIG_KEYS
 
    # Make sure we have the default values set when we get 'extended'
    def self.extended(base)
      base.reset
    end

    def configure
      yield self
    end

    def options
      Hash[ * VALID_CONFIG_KEYS.map { |key| [key, send(key)] }.flatten ]
    end
 
    def reset
      self.endpoint    = DEFAULT_ENDPOINT
      self.method      = DEFAULT_METHOD
      self.user_agent  = DEFAULT_USER_AGENT
 
      self.api_key     = DEFAULT_API_KEY
      self.format      = DEFAULT_FORMAT
      self.api_version = DEFAULT_API_VERSION
    end

  end
end